/************************************************************************************************
 * InfoIC.dll implementation for MiniPro v6.17 (http://autoelectric.cn/EN/TL866_main.html)
 *
 * Code is using ezXML : Copyright 2004-2006 Aaron Voisine <aaron@voisine.org>
 *
 * XpoZed; http://nullsecurity.org; 13.Sep.2015
 *
 */

#include "main.h"

/** GetIcStru() :: Get a IC structure by given Manufacturer ID and IC ID [Dead function]
*
* MfcID		:: Manufacturer ID
* ICID		:: IC ID
* *result	:: Result written as ICStru structure
*
* RetVal	:: always FALSE
*
*/
BOOL DLL_EXPORT GetIcStru(DWORD MfcID, DWORD ICID, byte *result) {
	/// Get the Manufacturer
	memcpy(&suMfc, (dataMFC + (MfcID * sizeof(MfcStru))), sizeof(MfcStru));

	/// Get the IC
	memcpy(result, (suMfc.mfc_ic_struc + (ICID * sizeof(ICStru))), sizeof(ICStru));

    return FALSE;
}

/** GetIcMFC() :: Get a list of Manufacturer IDs
 *
 * *search	:: User typed word into device name search field
 * *IDlist	:: result buffer, holding the affected Manufacturer IDs
 * type		:: IC type, taken from the radio boxes
 *
 * RetVal	:: Number of Manufacturers in the IDlist
 *
 */
DWORD DLL_EXPORT GetIcMFC(char *search, byte *IDlist, DWORD type) {
	DWORD curMfcID, curICID, countMfc;
	BOOL doSearch;

	doSearch = FALSE;
	if (strlen(search) > 0)  {
		doSearch = TRUE;
	}

	/// Build array of Manufacturer IDs
	countMfc = 0;
	for(curMfcID = 0; curMfcID < countMFC; curMfcID++) {
		/// Fill in the MfcStruc
		memcpy(&suMfc, (dataMFC + (curMfcID * sizeof(MfcStru))), sizeof(MfcStru));
		/// Iterate through manufacturer ICs
		for(curICID = 0; curICID < suMfc.mfc_ic_count; curICID++) {
			/// Fill in the ICStruc
			memcpy(&suIC, (suMfc.mfc_ic_struc + (curICID * sizeof(ICStru))), sizeof(ICStru));

			/// If the type is equal to the user choice, add the IC to ICs ID list
			if ((type == 0) || (type == suIC._IC_Type)) {
				if (doSearch == TRUE) {
					/// User entered a search word
					if (StrStr(suIC._IC_Name, search) != NULL) {
						*(DWORD*)&IDlist[countMfc*4] = curMfcID;
						countMfc++;
						break;
					}
				} else {
					*(DWORD*)&IDlist[countMfc*4] = curMfcID;
					countMfc++;
					break;
				}
			}
		}
	}
    return countMfc;
}

/** GetIcList() :: Get a list of IC for chosen Manufacturer
 *
 * *search	:: User typed word into device name search field
 * *IDlist	:: result buffer, holding the affected IC IDs
 * id		:: Manufacturer ID
 * type		:: IC type, taken from the radio boxes
 *
 * RetVal	:: Number of ICs in the IDlist
 *
 */
DWORD DLL_EXPORT GetIcList(char *search, byte *IDlist, DWORD id, DWORD type) {
	int i, countIC;
	BOOL doSearch;

	doSearch = FALSE;
	if (strlen(search) > 0)  {
		doSearch = TRUE;
	}

	/// Get the requested manufacturer data
	memcpy(&suMfc, (dataMFC + (id * sizeof(MfcStru))), sizeof(MfcStru));

	/// if the manufacturer contains no ICs, return 0
	if (suMfc.mfc_ic_count <= 0) {
		return 0;
	}

	/// Build array of IC IDs
	countIC = 0;
	for(i = 0; i < suMfc.mfc_ic_count; i++) {
		/// Fill in the ICStruc
		memcpy(&suIC, (suMfc.mfc_ic_struc + (i * sizeof(ICStru))), sizeof(ICStru));
		if ((type == 0) || (type == suIC._IC_Type)) {
			if (doSearch == TRUE) {
				/// User entered a search word
				if (StrStr(suIC._IC_Name, search) != NULL) {
					*(DWORD*)&IDlist[countIC*4] = i;
					countIC++;
				}
			} else {
				*(DWORD*)&IDlist[countIC*4] = i;
				countIC++;
			}
		}
	}

    return countIC;
}

/** GetMfcStru() :: Get a Manufacturer structure by given ID
 *
 * id		:: Manufacturer ID
 * *result	:: Result written as MfcStru structure
 *
 * RetVal	:: none
 *
 */
DWORD DLL_EXPORT GetMfcStru(DWORD id, byte *result) {
	memcpy(result, (dataMFC + (id * sizeof(MfcStru))), sizeof(MfcStru));
	return 0;
}

/** GetDllInfo() :: Get the total number of ICs in the database
 *
 * RetVal	:: Total number of ICs in the database
 *
 */
DWORD DLL_EXPORT GetDllInfo(DWORD *arg1, DWORD *arg2) {
	*arg1 = 0x64;
	*arg2 = countMFC;

	/// Since we get the IC's total at DLL load, we can just return the value
	return countIC;
}

/** getDWORD() :: Just a wrapper
 *
 * hNode	:: XML node handle
 * *attr	:: Attribute name
 *
 * RetVal	:: Hexadecimal string converted to DWORD value
 *
 */
DWORD getDWORD(ezxml_t hNode, char *attr) {
	return strtoul(ezxml_attr(hNode, attr), (char**)0, 0);
}

/** init() :: initalize the XML data and build structure in memory
 *
 */
BOOL init() {
	HANDLE hFile;
	ezxml_t hChipDB, manufacturer, ic, ic_memory, ic_settings;

	hFile = CreateFile(szChipDB, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (hFile == INVALID_HANDLE_VALUE) {
		char *errMsg = VirtualAlloc(NULL, MAX_PATH + strlen(szChipDB), MEM_COMMIT|MEM_RESERVE, PAGE_READWRITE);
		wsprintf(errMsg, "Unable to open \"%s\" for reading.\nVerify that \"%s\" is located in InfoDB.dll directory.\n\nError #0x%08X", szChipDB, szChipDB, GetLastError());
		MessageBox(NULL, errMsg, "InfoDB.dll error", MB_OK|MB_ICONSTOP);
		VirtualFree(errMsg, 0, MEM_RELEASE);
		return FALSE;
	}
	CloseHandle(hFile);

	/// hChipDB is global, don't worry about it
	hChipDB = ezxml_parse_file(szChipDB);

	/// lets parse the XML and fill the memory with it
	/// Count the manufacturer and IC totals, so we can allocate only 2 buffers, instead of tens of thousands...
	countMFC = 0;	/// Zero the total MFC counter
	countIC = 0;	/// Zero the total IC counter
	for (manufacturer = ezxml_child(hChipDB, "manufacturer"); manufacturer; manufacturer = manufacturer->next) {
		for (ic = ezxml_child(manufacturer, "ic"); ic; ic = ic->next) {
			countIC++;
		}
		countMFC++;
	}

	/// at this point, we have the number of Manufacturers and ICs, so we can allocate the buffers
	if (countMFC > 0) {
		dataMFC = dataMFC_t = VirtualAlloc(NULL, (countMFC*sizeof(MfcStru)), MEM_COMMIT|MEM_RESERVE, PAGE_READWRITE);
		dataIC = dataIC_t = VirtualAlloc(NULL, (countIC*sizeof(ICStru)), MEM_COMMIT|MEM_RESERVE, PAGE_READWRITE);

		suMfc.mfc_id = 0;
		for (manufacturer = ezxml_child(hChipDB, "manufacturer"); manufacturer; manufacturer = manufacturer->next) {
			/// Fill in the Manufacturer structure
			suMfc.mfc_logo = atol(ezxml_attr(manufacturer, "logo"));
			lstrcpy(suMfc.mfc_name_short, ezxml_attr(manufacturer, "name_short"));
			lstrcpy(suMfc.mfc_name_full, ezxml_attr(manufacturer, "name_full"));
			suMfc.mfc_ic_struc = dataIC_t;
			suMfc.mfc_ic_count = 0;

			for (ic = ezxml_child(manufacturer, "ic"); ic; ic = ic->next) {
				ic_memory = ezxml_child(ic, "memory");
				ic_settings = ezxml_child(ic, "settings");

				/// Fill in the IC structure
				suIC._IC_u01 = getDWORD(ic_settings, "u01");
				suIC._IC_u02 = getDWORD(ic_settings, "u02");
				suIC._IC_Type = getDWORD(ic, "type");
				lstrcpy(suIC._IC_Name, ezxml_attr(ic, "name"));
				suIC._IC_u03 = getDWORD(ic_settings, "u03");
				suIC._IC_CodeMemory = getDWORD(ic_memory, "code");
				suIC._IC_DataMemory = getDWORD(ic_memory, "data");
				suIC._IC_UserMemory = getDWORD(ic_memory, "user");
				suIC._IC_u04 = (WORD)getDWORD(ic_settings, "u04");
				suIC._IC_u05 = (WORD)getDWORD(ic_settings, "u05");
				suIC._IC_u06 = (WORD)getDWORD(ic_settings, "u06");
				suIC._IC_u07 = (byte)getDWORD(ic_settings, "u07");
				suIC._IC_u08 = (byte)getDWORD(ic_settings, "u08");
				suIC._IC_u09 = getDWORD(ic_settings, "u09");
				suIC._IC_u10 = getDWORD(ic_settings, "u10");
				suIC._IC_ChipID = getDWORD(ic, "id");
				suIC._IC_u11 = getDWORD(ic_settings, "u11");
				suIC._IC_u12 = getDWORD(ic_settings, "u12");
				suIC._IC_u13 = getDWORD(ic_settings, "u13");
				suIC._IC_Package = getDWORD(ic, "package");
				suIC._IC_u14 = getDWORD(ic_settings, "u14");

				/// copy the IC structure to its final destination buffer
				memcpy(dataIC_t, &suIC, sizeof(ICStru));

				/// move the offset to the next IC structure
				dataIC_t = dataIC_t + sizeof(ICStru);

				/// We need this counter
				suMfc.mfc_ic_count++;
			}

			/// copy the Mfc structure to its final destination buffer
			memcpy(dataMFC_t, &suMfc, sizeof(MfcStru));

			/// move the offset to the next IC structure
			dataMFC_t = dataMFC_t + sizeof(MfcStru);
			suMfc.mfc_id++;
		}
	}

	/// It seems that ezXML has problems with the garbage cleaner, so until I bugfix it, there will be no memory release
	/// although it's a bad practice, the memory allocation is done only once (at program execution) and we can _HOPE_,
	/// that the OS will clean it (on program exit).
	/// Also, what's the worse that can happen, right? right?
	//ezxml_free(hChipDB);

	return TRUE;
}

/** clean() :: Clean buffers before library unload
 *
 */
void clean() {
	/// Free the Manufacturers buffer, if present
	if (dataMFC != NULL) {
		VirtualFree(dataMFC, 0, MEM_RELEASE);
	}
	/// Free the IC buffer, if present
	if (dataIC != NULL) {
		VirtualFree(dataIC, 0, MEM_RELEASE);
	}
}

/** DllMain()
 *
 */
BOOL APIENTRY DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved) {
	switch (fdwReason) {
		case DLL_PROCESS_ATTACH:
			return init();
		break;
		case DLL_PROCESS_DETACH:
			clean();
		break;
        case DLL_THREAD_ATTACH: break;
		case DLL_THREAD_DETACH: break;
	}
	return TRUE;
}
